@@ -8,7 +8,7 @@ use File::Basename;
 
 use vars qw(@ISA $VERSION);
 @ISA     = qw(MP3::Daemon);
-$VERSION = 0.09;
+$VERSION = 0.10;
 
 # constructor that does NOT daemonize itself
 #_______________________________________
@@ -48,6 +48,12 @@ use constant URL   => 0;
 use constant TITLE => 1;
 use constant TIME  => 2;
 
+# Audio::Play::MPG123 states
+#_______________________________________
+use constant STOPPED => 0;
+use constant PAUSED  => 0;
+use constant PLAYING => 0;
+
 # |>
 #_______________________________________
 sub play {
@@ -124,15 +130,17 @@ sub prev {
 # ==
 #_______________________________________
 sub pause {
-    my $self = shift;
-    $self->{player}->pause;
+    my $self   = shift;
+    my $player = $self->{player};
+    $player->pause() unless $player->state() == STOPPED;
 }
 
 # []
 #_______________________________________
 sub stop {
-    my $self = shift;
-    $self->{player}->stop;
+    my $self   = shift;
+    my $player = $self->{player};
+    $player->stop() unless $player->state == STOPPED;
 }
 
 # !!
@@ -362,12 +370,13 @@ sub rand {
 sub random {
     my $self = shift;
     my $pl   = $self->{playlist};
-    my $n    = scalar @$pl;
+    my $len  = scalar @$pl;
+    my $n;
 
-    if ($n) {
+    if ($len) {
 
         # prevent an mp3 from being played twice in a row
-        do { $n = int(rand($n)) } until ($n != $self->{n});
+        do { $n = int(rand($len)) } until ($n != $self->{n});
 
         $self->{n} = $n;
         $self->{player}->load($pl->[$n][URL]);
@@ -448,11 +457,15 @@ as C<$socket_path> in the following descriptions.
 
 =over 4
 
-=item new $socket_path 
+=item new (socket_path => $socket_path, at_exit => $code_ref)
 
-This instantiates a new MP3::Daemon::Simple.
+This instantiates a new MP3::Daemon.  The parameter, C<socket_path> is
+mandatory, but C<at_exit> is optional.
 
-    my $mp3d = MP3::Daemon::Simple->new($socket_path);
+    my $mp3d = MP3::Daemon::Simple->new (
+        socket_path => "$ENV{HOME}/.mp3/mp3_socket"
+        at_exit     => sub { print "farewell\n" },
+    );
 
 =item main
 
@@ -462,14 +475,17 @@ method will never return.
 
     $mp3d->main;
 
-=item spawn $socket_path 
+=item spawn (socket_path => $socket_path, at_exit => $code_ref)
 
 This combines C<new()> and C<main()> while also forking itself into
 the background.  The spawn method will return immediately to the
-parent process while the child process becomes an MP3::Daemon::Simple
-that is waiting for client requests.
+parent process while the child process becomes an MP3::Daemon that is
+waiting for client requests.
 
-    MP3::Daemon::Simple->spawn($socket_path);
+    MP3::Daemon::Simple->spawn (
+        socket_path => "$ENV{HOME}/.mp3/mp3_socket"
+        at_exit     => sub { print "farewell\n" },
+    );
 
 =item client $socket_path 
 
@@ -478,6 +494,40 @@ communicate with a previously instantiated MP3::Daemon::Simple.
 
     my $client = MP3::Daemon::Simple->client($socket_path);
 
+=item idle $code_ref
+
+This method has 2 purposes.  When called with a parameter that is a
+code reference, the purpose of this method is to specify a code reference
+to execute during times of idleness.  When called with no parameters,
+the specified code reference will be invoked w/ an MP3::Daemon object
+passed to it as its only parameter.  This method will be invoked
+at regular intervals while main() runs.
+
+B<Example>:  Go to the next song when there are 8 or fewer seconds left
+in the current mp3.
+
+    $mp3d->idle (
+        sub {
+            my $self   = shift;             # M:D:Simple
+            my $player = $self->{player};   # A:P:MPG123
+            my $f      = $player->{frame};  # hashref w/ time info
+
+            $self->next() if ($f->[2] <= 8);
+        }
+    );
+
+This is a flexible mechanism for adding additional behaviours during
+playback.
+
+=item atExit $code_ref
+
+This mimics the C function atexit().  It allows one to give an MP3::Daemon
+some CODEREFs to execute when the destructor is called.  Like the C version,
+the CODEREFs will be called in the reverse order of their registration.
+Unlike the C version, C<$self> will be given as a parameter to each CODEREF.
+
+    $mp3d->atExit( sub { unlink("$ENV{HOME}/.mp3/mp3.pid") } );
+
 =back
 
 =head2 Client Protocol
@@ -630,4 +680,4 @@ mpg123(1), Audio::Play::MPG123(3pm), pimp(1p), mpg123sh(1p), mp3(1p)
 
 =cut
 
-# $Id: Simple.pm,v 1.10 2001/07/23 16:18:06 beppu Exp $
+# $Id: Simple.pm,v 1.11 2001/07/25 22:58:16 beppu Exp $
